@@ -15,6 +15,8 @@
 ## along with this program; if not, write to the Free Software
 ## Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
+import sys
+import inspect
 import os
 import popen2
 import signal
@@ -48,6 +50,28 @@ class Puppy:
 		
 		return
 
+	def delete(self, filename):
+		args = ['-c', 'delete', filename]
+		
+		output_file = self._execute(args)
+
+		output = output_file.readlines()
+		output_file.close()
+		
+		print output
+		
+		if self.getStatus() != 0:
+			self._handleErrorOuput(output)
+		
+		return
+
+	def exists(self):
+		for path in os.environ['PATH'].split(':'):
+			if len(path) > 0 and os.access(path + '/puppy', os.F_OK):
+				return True
+			
+		return False
+		
 	def getDiskSpace(self):
 		args = ['-c', 'size']
 
@@ -57,7 +81,7 @@ class Puppy:
 		output_file.close()
 
 		if self.getStatus() != 0:
-			raise PuppyError("getDiskSpace() failed. puppy returned: " + str(output))
+			self._handleErrorOuput(output)
 
 		# Skip Total size line
 		total = float(output[0].split()[5]) * 1024 * 1024 * 1024
@@ -79,36 +103,7 @@ class Puppy:
 
 		return total, free_space
 
-		
-	def listDir(self, path=None):
-		args = ['-c', 'dir']
-		if path != None:
-			args.append(path)
-			
-		output_file = self._execute(args)
-
-		output = output_file.readlines()
-		output_file.close()
-
-		listing = []
-		# Parse output of output_file and return it as a list
-		for line in output:
-			entry = line.split()
-			space = ' '
-			item = [ entry[0], space.join(entry[7:]),
-			         "%s %s %s %s" % (entry[2], entry[3], entry[4], entry[6]),
-			         entry[1] ]
-			listing.append(item)
-		
-		status = self.getStatus()	
-		if status == Puppy.E_GLOBAL_LOCK:
-			raise PuppyBusyError(str(output))
-		elif status != 0:
-			raise PuppyError("listDir failed. puppy returned: " + str(output))
-		
-		return listing
-		
-		# FIXME: Can getFile() be merged with putFile()
+	# FIXME: Can getFile() be merged with putFile()
 	def getFile(self, src_file, dest_file=None):
 		args = ['-c', 'get', src_file]
 		if dest_file != None:
@@ -120,72 +115,13 @@ class Puppy:
 
 		status = self.getStatus(wait=False)
 		if status != 0 and status != -1:
-			raise PuppyError("getFile failed. puppy returned: " + str(self.progress_output))
-
-		return
-
-	def putFile(self, src_file, dest_file=None):
-		args = ['-c', 'put', src_file]
-		if dest_file != None:
-			args.append(dest_file)
-		else:
-			args.append(os.path.basename(src_file))
+			self._handleErrorOuput(self.progress_output)
 			
-		self.progress_output = self._execute(args)
-		
-		status = self.getStatus(wait=False)
-		if status != 0 and status != -1:
-			raise PuppyError("putFile failed. puppy returned: " + str(self.progress_output))
-			
-		return
-
-	def makeDir(self, dirname):
-		args = ['-c', 'mkdir', dirname]
-			
-		output_file = self._execute(args)
-
-		output = output_file.readlines()
-		output_file.close()
-		
-		# Parse output of output_file and return it as a list
-		
-		if self.getStatus() != 0:
-			raise PuppyError("makeDir failed. puppy returned: " + str(output))
-		
-		return
-
-	def rename(self, old_name, new_name):
-		args = ['-c', 'rename', old_name, new_name]
-			
-		output_file = self._execute(args)
-
-		output = output_file.readlines()
-		output_file.close()
-		
-		# Parse output of output_file and return it as a list
-		
-		if self.getStatus() != 0:
-			raise PuppyError("rename failed. puppy returned: " + str(output))
-		
-		return
-
-	def delete(self, filename):
-		args = ['-c', 'delete', filename]
-			
-		output_file = self._execute(args)
-
-		output = output_file.readlines()
-		output_file.close()
-		
-		# Parse output of output_file and return it as a list
-		
-		if self.getStatus() != 0:
-			raise PuppyError("delete failed. puppy returned: " + str(output))
-		
 		return
 
 	def getProgress(self):
 		exit_status = self.getStatus(wait=False)
+		
 		# exit_status of -1 means process is still alive
 		if exit_status != -1:
 			# Raise exception if puppy did not exit successfully or because of
@@ -212,7 +148,10 @@ class Puppy:
 		
 		if len(tokens) != 4:
 			# Reap child process
-			self.getStatus()
+			exit_status = self.getStatus()
+			if exit_status != 0 and exit_status != 15:
+				self._handleErrorOuput(line)
+			
 			return None, None, None
 
 		percent = tokens[0][:tokens[0].rindex('%')]
@@ -236,6 +175,98 @@ class Puppy:
 			
 		return status
 		
+	def listDir(self, path=None):
+		args = ['-c', 'dir']
+		if path != None:
+			args.append(path)
+			
+		output_file = self._execute(args)
+
+		output = output_file.readlines()
+		output_file.close()
+
+		status = self.getStatus()	
+		if status == Puppy.E_GLOBAL_LOCK:
+			raise PuppyBusyError(str(output))
+		elif status != 0:
+			self._handleErrorOuput(output)
+		
+		listing = []
+		# Parse output of output_file and return it as a list
+		for line in output:
+			entry = line.split()
+			space = ' '
+			item = [ entry[0], space.join(entry[7:]),
+			         "%s %s %s %s" % (entry[2], entry[3], entry[4], entry[6]),
+			         entry[1] ]
+			listing.append(item)
+		
+		return listing
+		
+	def makeDir(self, dirname):
+		args = ['-c', 'mkdir', dirname]
+			
+		output_file = self._execute(args)
+
+		output = output_file.readlines()
+		output_file.close()
+		
+		# XXX: Remove debug output
+		print output
+		
+		if self.getStatus() != 0:
+			self._handleErrorOuput(output)
+		
+		return
+
+	def putFile(self, src_file, dest_file=None):
+		args = ['-c', 'put', src_file]
+		if dest_file != None:
+			args.append(dest_file)
+		else:
+			args.append(os.path.basename(src_file))
+			
+		self.progress_output = self._execute(args)
+		
+		status = self.getStatus(wait=False)
+		if status != 0 and status != -1:
+			self._handleErrorOuput(self.progress_output)
+			
+		return
+
+	def rename(self, old_name, new_name):
+		args = ['-c', 'rename', old_name, new_name]
+
+		print 'args = ', args
+		output_file = self._execute(args)
+
+		output = output_file.readlines()
+		output_file.close()
+
+		# XXX: Remove debug output
+		print output
+		
+		if self.getStatus() != 0:
+			self._handleErrorOuput(output)
+		
+		return
+
+	def reset(self):	
+		args = ['-c', 'cancel']
+		
+		try:	
+			output_file = self._execute(args)
+		except PuppyBusyError:
+			return False
+
+		output = output_file.readlines()
+		output_file.close()
+		
+		if self.getStatus() != 0:
+			return False
+		
+		return True
+
 	def setTurbo(self, value):
 		args = ['-c', 'turbo']
 			
@@ -250,23 +281,10 @@ class Puppy:
 		output_file.close()
 		
 		if self.getStatus() != 0:
-			raise PuppyError("setTurbo failed. puppy returned: " + str(output))
+			self._handleErrorOuput(output)
 		
 		return
 			
-	def reset(self):	
-		args = ['-c', 'cancel']
-			
-		output_file = self._execute(args)
-
-		output = output_file.readlines()
-		output_file.close()
-		
-		if self.getStatus() != 0:
-			return False
-		
-		return True
-
 	def _anotherPuppyActive(self):
 		lock_file = open(Puppy.LOCK_FILE, 'a')
 		
@@ -302,15 +320,42 @@ class Puppy:
 			
 		return self.popen_obj.fromchild
 
+	def _handleErrorOuput(self, output):
+		"""Parse puppy error message and raise appropriate exception.
+		
+		"""
+		caller = sys._getframe(1)
+		try:
+			func_name = inspect.getframeinfo(caller)[2]
+		finally:
+			del caller
+
+		errmsg = ' '.join(output)
+		msg = func_name + '(): ' + errmsg
+		
+		if errmsg == 'ERROR: Can not autodetect a Topfield TF5000PVRt\n':
+			raise PuppyNoPVRError(msg)
+		else:
+			raise PuppyError(msg)
+		
 class PuppyError(Exception):
 	def __init__(self, value):
 		self.value = value
 	def __str__(self):
-		return repr(self.value)
+		return self.value
 
-class PuppyBusyError(Exception):
+class PuppyBusyError(PuppyError):
+	"""Exception raised for when another instance of Puppy is running.
+	
+	"""
+	def __init__(self, value):	
+		PuppyError.__init__(self, value)
+
+class PuppyNoPVRError(PuppyError):
+	"""Excpetion raised when no PVR is detected.
+	
+	"""
 	def __init__(self, value):
-		self.value = value
-	def __str__(self):
-		return repr(self.value)
+		msg = _('PVR not connected. Please check that your computer is connected to the PVR.')
+		PuppyError.__init__(self, msg)
 	
